!------------------------------------------------------------
subroutine integra_1(dim_par,system_dim,num_neurons,matriz_par,matriz_acopla_electrical,matriz_x,param_elec_coupling,matriz_acopla_chemical,dt)
!1-stage Euler method
implicit none
integer(kind=4),intent(in)	:: dim_par,system_dim,num_neurons,matriz_acopla_electrical(num_neurons,num_neurons),matriz_acopla_chemical(num_neurons,num_neurons)
real(kind=8),intent(in)		:: matriz_par(dim_par,num_neurons),param_elec_coupling,dt
real(kind=8),intent(inout)	:: matriz_x(system_dim,num_neurons)
real(kind=8)			:: matriz_x_tmp(system_dim,num_neurons),tra_x(system_dim,num_neurons),kk1(system_dim,num_neurons)
integer(kind=4)			:: i,j

tra_x(:,:)=matriz_x(:,:)

call calc_k(dim_par,system_dim,num_neurons,matriz_par,matriz_acopla_electrical,tra_x,kk1,param_elec_coupling,matriz_acopla_chemical)

matriz_x(:,:)=matriz_x(:,:)+dt*kk1(:,:)

return
end subroutine integra_1

!------------------------------------------------------------

subroutine integra_2(dim_par,system_dim,num_neurons,matriz_par,matriz_acopla_electrical,matriz_x,param_elec_coupling,matriz_acopla_chemical,dt)
!2-stage Heun method (or improved Euler method)
implicit none
integer(kind=4),intent(in)	:: dim_par,system_dim,num_neurons,matriz_acopla_electrical(num_neurons,num_neurons),matriz_acopla_chemical(num_neurons,num_neurons)
real(kind=8),intent(in)		:: matriz_par(dim_par,num_neurons),param_elec_coupling,dt
real(kind=8),intent(inout)	:: matriz_x(system_dim,num_neurons)
real(kind=8)			:: matriz_x_tmp(system_dim,num_neurons),tra_x(system_dim,num_neurons),kk1(system_dim,num_neurons),kk2(system_dim,num_neurons)
integer(kind=4)			:: i,j

tra_x(:,:)=matriz_x(:,:)

call calc_k(dim_par,system_dim,num_neurons,matriz_par,matriz_acopla_electrical,tra_x,kk1,param_elec_coupling,matriz_acopla_chemical)

tra_x(:,:)=matriz_x(:,:)+kk1(:,:)*dt

call calc_k(dim_par,system_dim,num_neurons,matriz_par,matriz_acopla_electrical,tra_x,kk2,param_elec_coupling,matriz_acopla_chemical)

matriz_x(:,:)=matriz_x(:,:)+dt*(kk1(:,:)+kk2(:,:))/2.d0

return
end subroutine integra_2

!------------------------------------------------------------

subroutine integra_3(dim_par,system_dim,num_neurons,matriz_par,matriz_acopla_electrical,matriz_x,param_elec_coupling,matriz_acopla_chemical,dt)
!2-stage Arithmetic Mean Runge - Kutta method (3rd order)
implicit none
integer(kind=4),intent(in)	:: dim_par,system_dim,num_neurons,matriz_acopla_electrical(num_neurons,num_neurons),matriz_acopla_chemical(num_neurons,num_neurons)
real(kind=8),intent(in)		:: matriz_par(dim_par,num_neurons),param_elec_coupling,dt
real(kind=8),intent(inout)	:: matriz_x(system_dim,num_neurons)
real(kind=8)			:: matriz_x_tmp(system_dim,num_neurons),tra_x(system_dim,num_neurons),kk1(system_dim,num_neurons),kk2(system_dim,num_neurons),kk3(system_dim,num_neurons)
integer(kind=4)			:: i,j

tra_x(:,:)=matriz_x(:,:)

call calc_k(dim_par,system_dim,num_neurons,matriz_par,matriz_acopla_electrical,tra_x,kk1,param_elec_coupling,matriz_acopla_chemical)

tra_x(:,:)=matriz_x(:,:)+0.5d0*kk1(:,:)*dt

call calc_k(dim_par,system_dim,num_neurons,matriz_par,matriz_acopla_electrical,tra_x,kk2,param_elec_coupling,matriz_acopla_chemical)

tra_x(:,:)=matriz_x(:,:)+kk1(:,:)*dt

call calc_k(dim_par,system_dim,num_neurons,matriz_par,matriz_acopla_electrical,tra_x,kk3,param_elec_coupling,matriz_acopla_chemical)

matriz_x(:,:)=matriz_x(:,:)+dt*(kk1(:,:)+4.d0*kk2(:,:)+kk3(:,:))/6.d0

return
end subroutine integra_3

!------------------------------------------------------------

subroutine integra_4(dim_par,system_dim,num_neurons,matriz_par,matriz_acopla_electrical,matriz_x,param_elec_coupling,matriz_acopla_chemical,dt)
!classic Runge - Kutta method (4th order)
implicit none
integer(kind=4),intent(in)	:: dim_par,system_dim,num_neurons,matriz_acopla_electrical(num_neurons,num_neurons),matriz_acopla_chemical(num_neurons,num_neurons)
real(kind=8),intent(in)		:: matriz_par(dim_par,num_neurons),param_elec_coupling,dt
real(kind=8),intent(inout)	:: matriz_x(system_dim,num_neurons)
real(kind=8)			:: matriz_x_tmp(system_dim,num_neurons),tra_x(system_dim,num_neurons),kk1(system_dim,num_neurons),kk2(system_dim,num_neurons),kk3(system_dim,num_neurons),kk4(system_dim,num_neurons)
integer(kind=4)			:: i,j

tra_x(:,:)=matriz_x(:,:)

call calc_k(dim_par,system_dim,num_neurons,matriz_par,matriz_acopla_electrical,tra_x,kk1,param_elec_coupling,matriz_acopla_chemical)

tra_x(:,:)=matriz_x(:,:)+0.5d0*kk1(:,:)*dt

call calc_k(dim_par,system_dim,num_neurons,matriz_par,matriz_acopla_electrical,tra_x,kk2,param_elec_coupling,matriz_acopla_chemical)

tra_x(:,:)=matriz_x(:,:)+0.5d0*kk2(:,:)*dt

call calc_k(dim_par,system_dim,num_neurons,matriz_par,matriz_acopla_electrical,tra_x,kk3,param_elec_coupling,matriz_acopla_chemical)

tra_x(:,:)=matriz_x(:,:)+kk3(:,:)*dt

call calc_k(dim_par,system_dim,num_neurons,matriz_par,matriz_acopla_electrical,tra_x,kk4,param_elec_coupling,matriz_acopla_chemical)

matriz_x(:,:)=matriz_x(:,:)+dt*(kk1(:,:)+2.d0*(kk2(:,:)+kk3(:,:))+kk4(:,:))/6.d0

return
end subroutine integra_4

!------------------------------------------------------------

subroutine calc_k(dim_par,system_dim,num_neurons,matriz_par,matriz_acopla_electrical,tra_x,kk,param_elec_coupling,matriz_acopla_chemical)
implicit none
integer(kind=4),intent(in)	:: dim_par,system_dim,num_neurons,matriz_acopla_electrical(num_neurons,num_neurons),matriz_acopla_chemical(num_neurons,num_neurons)
real(kind=8),intent(in)		:: matriz_par(dim_par,num_neurons),tra_x(system_dim,num_neurons),param_elec_coupling
real(kind=8),intent(inout)	:: kk(system_dim,num_neurons)
real(kind=8)			:: acoplamento_chemical,acoplamento_electrical
integer(kind=4)			:: col,i

do col=1,num_neurons !number of neurons
	acoplamento_electrical=0.d0
	acoplamento_chemical=0.d0
 	do i=1,num_neurons !number of neurons
		!electrical coupling
		!i ---> col (matriz_acopla_electrical)
		if (matriz_acopla_electrical(col,i)==1) acoplamento_electrical=acoplamento_electrical+param_elec_coupling*matriz_acopla_electrical(col,i)*(tra_x(1,i)-tra_x(1,col))
		!chemical coupling
		!i ---> col (matriz_acopla_chemical)
		if (matriz_acopla_chemical(col,i)==1) acoplamento_chemical=acoplamento_chemical+matriz_par(3,col)*matriz_acopla_chemical(col,i)*(1.d0/(1.d0+dexp(-10.d0*(tra_x(1,i)-matriz_par(6,col)))))	
 	enddo
	acoplamento_chemical=acoplamento_chemical*(tra_x(1,col)-matriz_par(2,col))

	!equations of the evolution of neurons (first three kk) and of phase (fourth kk)
	kk(1,col)=tra_x(2,col)+3.d0*tra_x(1,col)**2-tra_x(1,col)**3-tra_x(3,col)+matriz_par(1,col)+acoplamento_electrical-acoplamento_chemical !for p
	kk(2,col)=1.d0-5.d0*tra_x(1,col)**2-tra_x(2,col) !for q
	kk(3,col)=-matriz_par(4,col)*tra_x(3,col)+4.d0*matriz_par(4,col)*(tra_x(1,col)+1.6d0) !for n
	kk(4,col)=(kk(2,col)*tra_x(1,col)-kk(1,col)*tra_x(2,col))/(tra_x(1,col)**2+tra_x(2,col)**2) !for phase
enddo

return
end subroutine calc_k
!------------------------------------------------------------