#----------------------------------------------------------------------------------------------------------------
# Programmed by Dr. Chris Antonopoulos
# email: cantonop76@gmail.com
#----------------------------------------------------------------------------------------------------------------


#----------------------------------------------------------------------------------------------------------------
# How to run the R script: source('network_statistical_measures_calculation_igraph.R')
#----------------------------------------------------------------------------------------------------------------


#----------------------------------------------------------------------------------------------------------------
network_filename_initial='c-elegans_277.dat'
network_filename=substr(network_filename_initial,1,nchar(network_filename_initial)-4)
#----------------------------------------------------------------------------------------------------------------


#----------------------------------------------------------------------------------------------------------------
number_of_random_networks=100
num_nodes_threshold_plot=300

# if switch_igraph_plots==1 then igraph network plots will be computed and shown, else they will not 
switch_igraph_plots=0
#----------------------------------------------------------------------------------------------------------------


#----------------------------------------------------------------------------------------------------------------
# start the clock
ptm <- proc.time()
#----------------------------------------------------------------------------------------------------------------


#----------------------------------------------------------------------------------------------------------------
library(igraph)
#----------------------------------------------------------------------------------------------------------------


#----------------------------------------------------------------------------------------------------------------
x <- read.table(network_filename_initial)
net <- as.matrix(x)
mode(net) <- "numeric"
net_initial=net
source('my_image_plot.R')
net_rotated_90d <- t(net_initial)[,nrow(net_initial):1]
dev.new()
myImagePlot(net_rotated_90d,zlim=c(0,1),title=cbind(network_filename,'Adjacency matrix of the ORIGINAL NETWORK'))
g <- graph.adjacency(net,mode="undirected",weighted=NULL,add.rownames=NA,add.colnames=NA)
write.table(as.matrix(get.adjacency(g)),paste('igraph_network_',network_filename,'.txt',sep=""),sep=" ",row.names=FALSE,col.names=FALSE)
#----------------------------------------------------------------------------------------------------------------


#----------------------------------------------------------------------------------------------------------------
# degree and PDF degree calculation
d=degree(g)
if (switch_igraph_plots==1)
{
	if (length(d)<=num_nodes_threshold_plot)
	{
		# plot the network on screen
		dev.new()
		plot(g,layout=layout.fruchterman.reingold,vertex.size=3,frame=FALSE,main=cbind(network_filename,'ORIGINAL NETWORK'))
	}
}
dd=degree.distribution(g)
dd=dd[-1]
write(d,file=paste('degrees_',network_filename,'.txt',sep=""),ncolumns=1,sep="\t")
write(dd,file=paste('degree_distribution_',network_filename,'.txt',sep=""),ncolumns=1,sep="\t")
d_bins=seq(1,max(d),by=1)
dm_bins_dd=cbind(d_bins,dd)
# go through each row and determine if a value is zero
row_sub=apply(dm_bins_dd,1,function(row) all(row !=0 ))
# subset as usual
dm_bins_dd_non_zero=dm_bins_dd[row_sub,]
write.table(dm_bins_dd_non_zero,paste('PDF_degree_',network_filename,'.txt',sep=""),row.names=FALSE,col.names=FALSE)
dev.new()
plot(dm_bins_dd_non_zero,xlab='degree',ylab='degree distribution',pch=1,col=1,type="b",main=cbind(network_filename,'Degree distribution of the ORIGINAL NETWORK'))
#----------------------------------------------------------------------------------------------------------------


#----------------------------------------------------------------------------------------------------------------
# assortativity calculation --> caclulation of -1 <= r <= 1
r=assortativity.degree(g,directed=FALSE)
# creation of a random network of a given degree sequence
degrees_ordered_descending=d[order(-d)]
r_random_network <- vector(mode="numeric",length=number_of_random_networks)
for (i in 1: number_of_random_networks)
{
	random_network_g <- degree.sequence.game(degrees_ordered_descending)
	r_random_network[i]=assortativity.degree(random_network_g,directed=FALSE)
}
cat("The assortativity coefficient is the Pearson correlation coefficient of degree between pairs of linked nodes. Positive values of\nr indicate a correlation between nodes of similar degree, while negative values indicate relationships between nodes of different\ndegree. In general, r lies between −1 and 1. When r=1, the network is said to have perfect assortative mixing patterns, when r=0\nthe network is non-assortative, while at r=−1 the network is completely disassortative","\n",file=paste('inference_assortativity_r_coefficient_',network_filename,'.txt',sep=""))
cat("\n\n","---------","\n","Assortativity (r coefficient) of the ORIGINAL NETWORK=",r,file=paste('inference_assortativity_r_coefficient_',network_filename,'.txt',sep=""),append=TRUE)
cat("\n","---------","\n","Mean assortativity (mean r coefficient) of the",number_of_random_networks,"random networks=",mean(r_random_network),"\n","---------",file=paste('inference_assortativity_r_coefficient_',network_filename,'.txt',sep=""),append=TRUE)
if (r>0)
{
	cat("\n\n\n","---------","\n","FINAL INFERENCE: The ORIGINAL NETWORK is ASSORTATIVE!","\n ---------",file=paste('inference_assortativity_r_coefficient_',network_filename,'.txt',sep=""),append=TRUE)
}
if (r==0)
{
	cat("\n\n\n","---------","\n","FINAL INFERENCE: The ORIGINAL NETWORK is NON-ASSORTATIVE!","\n ---------",file=paste('inference_assortativity_r_coefficient_',network_filename,'.txt',sep=""),append=TRUE)
}
if (r<0)
{
	cat("\n\n\n","---------","\n","FINAL INFERENCE: The ORIGINAL NETWORK is DISASSORTATIVE!","\n ---------",file=paste('inference_assortativity_r_coefficient_',network_filename,'.txt',sep=""),append=TRUE)
}
#----------------------------------------------------------------------------------------------------------------


#----------------------------------------------------------------------------------------------------------------
# transitivity calculation (global clustering coefficient) --> 0 <= gcc <= 1
gcc=transitivity(g,'global')
cat("\n","---------","\n","Global clustering coefficient of the ORIGINAL NETWORK=",gcc,"\n","---------","\n",file=paste('global_clustering_coefficient_',network_filename,'.txt',sep=""))
#----------------------------------------------------------------------------------------------------------------


#----------------------------------------------------------------------------------------------------------------
# average degree knn of the neighbors of nodes with degree d
knnoutput=graph.knn(g)
d_vs_knnoutput=cbind(d_bins,knnoutput[[2]])
row_sub_knnoutput=d_vs_knnoutput[complete.cases(d_vs_knnoutput),] # delete Nan values
write.table(row_sub_knnoutput,paste('average_degree_knn_',network_filename,'.txt',sep=""),row.names=FALSE,col.names=FALSE)
dev.new()
plot(row_sub_knnoutput,xlim=c(0,max(d)),ylim=c(0,max(row_sub_knnoutput[,2])),xlab='degree',ylab='knn(degree)',pch=1,col=1,type="b",main=cbind(network_filename,'Average degree knn(d) of the neighbors of nodes with degree d of the ORIGINAL NETWORK'))
#----------------------------------------------------------------------------------------------------------------


#----------------------------------------------------------------------------------------------------------------
# transitivity calculation (local clustering coefficients) --> 0 <= lcc <= 1
lcc=transitivity(g,'local')
lcc_initial=lcc
lcc_initial_non_Nan=lcc_initial[complete.cases(lcc_initial)] # delete Nan values
node_set=seq(1,vcount(g),by=1)
d_vs_lcc=cbind(node_set,lcc)
d_vs_lcc=d_vs_lcc[complete.cases(d_vs_lcc),] # delete Nan values
write.table(d_vs_lcc,paste('local_clustering_coefficients_',network_filename,'.txt',sep=""),row.names=FALSE,col.names=FALSE)
dev.new()
plot(d_vs_lcc,xlim=c(0,max(d_vs_lcc)),pch=21,col='black',ylim=c(0,1),xlab='node',ylab='CC(node)',main=cbind(network_filename,'Clustering coefficient CC(node) of nodes with degree d of the ORIGINAL NETWORK'))
# fit a line to the CC(node) vs node plot
myline.fit=lm(d_vs_lcc[,2] ~ d_vs_lcc[,1])
# get information about the fit
#summary(myline.fit)
# draw the fit line on the plot
abline(myline.fit,col='red')
text(length(d)-length(d)/4,1,paste('slope=',round(myline.fit$coefficients[[2]],5)),col='red')

#----------------------------------------------------------------------------------------------------------------
# transitivity calculation (mean local clustering coefficient) --> 0 <= mlcc <= 1
mlcc=mean(lcc_initial_non_Nan)
cat("\n","---------","\n","Average of local clustering coefficients of the ORIGINAL NETWORK=",mlcc,"\n","---------","\n",file=paste('network_average_clustering_coefficients_',network_filename,'.txt',sep=""))
write.table(cbind(min(d_vs_lcc[,1]),mlcc),file=paste('network_average_clustering_coefficients_gnuplot_',network_filename,'.txt',sep=""),row.names=FALSE,col.names=FALSE)
write.table(cbind(max(d_vs_lcc[,1]),mlcc),file=paste('network_average_clustering_coefficients_gnuplot_',network_filename,'.txt',sep=""),row.names=FALSE,col.names=FALSE,append=TRUE)
#----------------------------------------------------------------------------------------------------------------

abline(h=mlcc,col='blue')
text(length(d)-length(d)/4,0.95,paste('<CC>=',round(mlcc,5)),col='blue')

lcc_vs_d=cbind(d,lcc)
lcc_vs_d=lcc_vs_d[complete.cases(lcc_vs_d),] # delete Nan values
lcc_vs_d_ordered=lcc_vs_d[order(lcc_vs_d[,1]),]
counter_groups_tmp=1
for (i in 1:length(lcc_vs_d_ordered[,1]))
{
	if (lcc_vs_d_ordered[[i+1]]!=lcc_vs_d_ordered[[i]]) # still in the same group of degree
	{
			counter_groups_tmp=counter_groups_tmp+1
	}
}
if (is.directed(g)==TRUE)
{
	counter_groups_tmp=counter_groups_tmp-1
}
tmpsum <- vector(mode="numeric",length=counter_groups_tmp)
tmpsumnew <- vector(mode="numeric",length=counter_groups_tmp)
counter_average <- vector(mode="numeric",length=counter_groups_tmp)
degree_group <- vector(mode="numeric",length=counter_groups_tmp)
for (i in 1:counter_groups_tmp)
{
	tmpsum[[i]]=0
	counter_average[[i]]=0
}
counter_groups=1
degree_group[1]=lcc_vs_d_ordered[1]
for (i in 1:length(lcc_vs_d_ordered[,1])) # go through all the degrees
{
	if (lcc_vs_d_ordered[[i+1]]==lcc_vs_d_ordered[[i]]) # still in the same group of degree
	{
			tmpsum[[counter_groups]]=tmpsum[[counter_groups]]+lcc_vs_d_ordered[[i,2]]
			counter_average[[counter_groups]]=counter_average[[counter_groups]]+1
	}
	else # enter new group of degrees
	{
		tmpsum[[counter_groups]]=tmpsum[[counter_groups]]+lcc_vs_d_ordered[[i,2]]
		tmpsumnew[[counter_groups]]=tmpsum[[counter_groups]]/(counter_average[[counter_groups]]+1)
		counter_groups=counter_groups+1
		degree_group[counter_groups]=lcc_vs_d_ordered[i+1]
	}
}
tmpsumnew_vs_knnoutput=cbind(degree_group,tmpsumnew)
# go through each row and determine if a value is zero
row_sub=apply(tmpsumnew_vs_knnoutput,1,function(row) all(row !=0 ))
# subset as usual
tmpsumnew_vs_knnoutput_non_zero= tmpsumnew_vs_knnoutput[row_sub,]
tmp=cbind(degree_group,counter_average+1)
tmp <- tmp[-length(degree_group),]
write.table(tmpsumnew_vs_knnoutput_non_zero,paste('mean_clustering_coefficient_of_nodes_with_degree_d_',network_filename,'.txt',sep=""),row.names=FALSE,col.names=FALSE)
write.table(tmp,paste('degree_groups_count_',network_filename,'.txt',sep=""),row.names=FALSE,col.names=FALSE)
dev.new()
plot(tmpsumnew_vs_knnoutput_non_zero,xlim=c(0,max(tmpsumnew_vs_knnoutput[,1])),ylim=c(0,max(tmpsumnew_vs_knnoutput[,2])),xlab='degree',ylab='CC(degree)',pch=1,col=1,type="b",main=cbind(network_filename,'Average clustering coefficient CC(d) of nodes with degree d of the ORIGINAL NETWORK'))
#----------------------------------------------------------------------------------------------------------------


#----------------------------------------------------------------------------------------------------------------
# mean shortest path calculation
mean_shortest_path=average.path.length(g)
cat("\n","---------","\n","Mean shortest path of the ORIGINAL NETWORK=",mean_shortest_path,"\n","---------","\n",file=paste('mean_shortest_path_',network_filename,'.txt',sep=""))
#----------------------------------------------------------------------------------------------------------------


#----------------------------------------------------------------------------------------------------------------
# creation of a random network of a given degree sequence
mean_shortest_path_random_network <- vector(mode="numeric",length=number_of_random_networks)
lcc_random_network <- vector(mode="numeric",length=number_of_random_networks)
for (i in 1: number_of_random_networks)
{
	random_network_g <- degree.sequence.game(degrees_ordered_descending)
	# mean shortest path calculation of the random network
	mean_shortest_path_random_network[i]=average.path.length(random_network_g)
	lcc_initial_random_network_g=transitivity(random_network_g,'local')
	lcc_initial_lcc_initial_random_network_g_non_Nan= lcc_initial_random_network_g[complete.cases(lcc_initial_random_network_g)] # delete Nan values
	lcc_random_network[i]=mean(lcc_initial_lcc_initial_random_network_g_non_Nan)
}
write(mean_shortest_path_random_network,file=paste('mean_shortest_paths_random_networks_',network_filename,'.txt',sep=""),ncolumns=1,sep='\t')
cat("---------","\n","Average of mean shortest paths of the above",number_of_random_networks,"random networks=",mean(mean_shortest_path_random_network),"\n",file=paste('mean_shortest_paths_random_networks_',network_filename,'.txt',sep=""),append=TRUE)
cat("---------","\n","Standard deviation of mean shortest path of the",number_of_random_networks,"random networks=",sd(mean_shortest_path_random_network),"\n","---------",file=paste('mean_shortest_paths_random_networks_',network_filename,'.txt',sep=""),append=TRUE)
cat("A network is considered small world, if the average clustering coefficient of the ORIGINAL NETWORK is significantly higher than a \nrandom network constructed on the same vertex set, and if the network has approximately the same mean-shortest path length as its \ncorresponding random network","\n",file=paste('inference_small_world_network_or_not_',network_filename,'.txt',sep=""))
cat("\n\n","---------","\n","Network average clustering coefficient of the ORIGINAL NETWORK=",mlcc,file=paste('inference_small_world_network_or_not_',network_filename,'.txt',sep=""),append=TRUE)
cat("\n","---------","\n","Mean of the network average clustering coefficient of the",number_of_random_networks,"random networks=",mean(lcc_random_network),file=paste('inference_small_world_network_or_not_',network_filename,'.txt',sep=""),append=TRUE)
#cat("\n","---------","\n","Absolute difference between the mean of the network average clustering coefficient of the ORIGINAL NETWORK and that of the",number_of_random_networks,"random \n networks=",abs(mlcc-mean(lcc_random_network))," <===== This number should be significantly BIG for the ORIGINAL NETWORK to be SMALL WORLD","\n","---------","\n\n",file=paste('inference_small_world_network_or_not_',network_filename,'.txt',sep=""),append=TRUE)
cat("\n","---------","\n","Mean shortest path of the ORIGINAL NETWORK=",mean_shortest_path,file=paste('inference_small_world_network_or_not_',network_filename,'.txt',sep=""),append=TRUE)
cat("\n","---------","\n","Average of mean shortest paths of the above",number_of_random_networks,"random networks=",mean(mean_shortest_path_random_network),"\n","---------","\n",file=paste('inference_small_world_network_or_not_',network_filename,'.txt',sep=""),append=TRUE)
gamma=mlcc/mean(lcc_random_network)
lambda=mean_shortest_path/mean(mean_shortest_path_random_network)
sigma=gamma/lambda
cat("\n","---------","\n","γ=",gamma,file=paste('inference_small_world_network_or_not_',network_filename,'.txt',sep=""),append=TRUE)
cat("\n","---------","\n","λ=",lambda,file=paste('inference_small_world_network_or_not_',network_filename,'.txt',sep=""),append=TRUE)
cat("\n","---------","\n","σ(=γ/λ)=",sigma,"\n","---------",file=paste('inference_small_world_network_or_not_',network_filename,'.txt',sep=""),append=TRUE)
if (sigma>1)
{
	cat("\n\n\n","---------","\n","FINAL INFERENCE: The ORIGINAL NETWORK is SMALL WORLD!","\n ---------",file=paste('inference_small_world_network_or_not_',network_filename,'.txt',sep=""),append=TRUE)
}
if (sigma<=1)
{
	cat("\n\n\n","---------","\n","FINAL INFERENCE: The ORIGINAL NETWORK is NOT SMALL WORLD!","\n ---------",file=paste('inference_small_world_network_or_not_',network_filename,'.txt',sep=""),append=TRUE)
}
d_random_network_g=degree(random_network_g)
dd_random_network_g=degree.distribution(random_network_g)
dd_random_network_g=dd_random_network_g[-1]
d_bins_random_network_g=seq(1,max(d_random_network_g),by=1)
dm_bins_dd_random_network_g=cbind(d_bins_random_network_g,dd_random_network_g)
# go through each row and determine if a value is zero
row_sub=apply(dm_bins_dd_random_network_g,1,function(row) all(row !=0 ))
# subset as usual
dm_bins_dd_random_network_g_non_zero=dm_bins_dd_random_network_g[row_sub,]
write.table(dm_bins_dd_random_network_g_non_zero,paste('PDF_degree_random_network_',network_filename,'.txt',sep=""),row.names=FALSE,col.names=FALSE)
dev.new()
plot(dm_bins_dd_random_network_g_non_zero,xlab='degree',ylab='degree distribution',pch=1,col=1,type="b",main=cbind(network_filename,'Degree distribution of a random network with the same degree sequence as the ORIGINAL NETWORK'))
#----------------------------------------------------------------------------------------------------------------


#----------------------------------------------------------------------------------------------------------------
# community detection
wtc <- walktrap.community(g,steps=6)
dput(wtc,file=paste('community_detection_details_',network_filename,'.txt',sep=""))
node_number=seq(1,length(d),by=1)
write.table(cbind(node_number,wtc[[3]]),paste('nodes_vs_communities_membership_',network_filename,'.txt',sep=""),row.names=FALSE,col.names=FALSE)
#----------------------------------------------------------------------------------------------------------------


#----------------------------------------------------------------------------------------------------------------
# computation of the modularity for the communities
ind <- wtc$membership+1
modcom=modularity(wtc)
cat("Modualrity Q is a measure of the extend to which like is connected to like in a network.\n0 < Q < 1\nQ takes positive values if there are more edges between vertices of the same type than we would expect by chance, and negative ones\nif there are less. If Q indicates positive value then the ORIGINAL NETWORK is ASSORTATIVE and if negative DISASSORTATIVE","\n",file=paste('inference_modularity_Q_coefficient_',network_filename,'.txt',sep=""))
cat("\n\n","---------","\n","Modularity (Q coefficient) of the ORIGINAL NETWORK=",modcom,"\n","---------",file=paste('inference_modularity_Q_coefficient_',network_filename,'.txt',sep=""),append=TRUE)
#if (modcom>0)
#{
#	cat("\n\n\n","---------","\n","FINAL INFERENCE: The ORIGINAL NETWORK is ASSORTATIVE!","\n"," ---------",file=paste('inference_modularity_Q_coefficient_',network_filename,'.txt',sep=""),append=TRUE)
#}
#if (modcom<0)
#{
#	cat("\n\n\n","---------","\n","FINAL INFERENCE: The ORIGINAL NETWORK is DISASSORTATIVE!","\n","---------",file=paste('inference_modularity_Q_coefficient_',network_filename,'.txt',sep=""),append=TRUE)
#}
#dev.new()
#postscript(paste(network_filename,".eps"),height=600,width=800)
tiff(paste(network_filename,".tif",sep=""),height=600,width=800)
par(cex.main=1.5)
plot(wtc,g,vertex.size=4,vertex.label=NA,edge.arrow.size=0.2,main=paste(network_filename,"\n","Walktrap Communities",sep=""))
text(1.7,1.26,paste("a)"),cex=2.5)
dev.off()
system(paste("convert ",paste(network_filename,".tif",sep="")," ",paste(network_filename,".eps",sep=""),sep=""))
#----------------------------------------------------------------------------------------------------------------







#----------------------------------------------------------------------------------------------------------------
# final report of all relevant quantities of the analysis
cat("\n","---------","\n","Final report of all relevant quantities of the analysis for the ORIGINAL NETWORK file:",network_filename_initial,"\n","---------","\n",file=paste('FINAL_REPORT_',network_filename,'.txt',sep=""))

today <- Sys.Date()
time <- Sys.time()

cat("\n","---------","\n","Date of execution:",format(today,format="%B %d %Y"),"\n",file=paste('FINAL_REPORT_',network_filename,'.txt',sep=""),append=TRUE)
cat(" Time of execution:",format(time,format="%H:%M:%S (Hours:Minute:Seconds)"),"\n","---------","\n",file=paste('FINAL_REPORT_',network_filename,'.txt',sep=""),append=TRUE)
if (is.directed(g)==TRUE)
{
	cat("\n","---------","\n","The ORIGINAL NETWORK is DIRECTED","\n","---------","\n",file=paste('FINAL_REPORT_',network_filename,'.txt',sep=""),append=TRUE)
	cat("\n","---------","\n","Number of nodes of the ORIGINAL NETWORK=",vcount(g),"\n","---------","\n",file=paste('FINAL_REPORT_',network_filename,'.txt',sep=""),append=TRUE)
	cat("\n","---------","\n","Number of links of the ORIGINAL NETWORK=",ecount(g),"\n","---------","\n",file=paste('FINAL_REPORT_',network_filename,'.txt',sep=""),append=TRUE)
}
if (is.directed(g)==FALSE)
{
	cat("\n","---------","\n","The ORIGINAL NETWORK is UNDIRECTED","\n","---------","\n",file=paste('FINAL_REPORT_',network_filename,'.txt',sep=""),append=TRUE)	
	cat("\n","---------","\n","Number of nodes of the ORIGINAL NETWORK=",vcount(g),"\n","---------","\n",file=paste('FINAL_REPORT_',network_filename,'.txt',sep=""),append=TRUE)
	cat("\n","---------","\n","Number of links of the ORIGINAL NETWORK=",2*ecount(g),"\n","---------","\n",file=paste('FINAL_REPORT_',network_filename,'.txt',sep=""),append=TRUE)
}
cat("\n","---------","\n","Number of random networks used in the analysis=",number_of_random_networks,"\n","---------","\n",file=paste('FINAL_REPORT_',network_filename,'.txt',sep=""),append=TRUE)
cat("\n","---------","\n","Global clustering coefficient of the ORIGINAL NETWORK=",gcc,"\n","---------","\n",file=paste('FINAL_REPORT_',network_filename,'.txt',sep=""),append=TRUE)
cat("\n","---------","\n","Average of local clustering coefficients of the ORIGINAL NETWORK=",mlcc,"\n","---------","\n",file=paste('FINAL_REPORT_',network_filename,'.txt',sep=""),append=TRUE)
cat("\n","---------","\n","Mean shortest path of the ORIGINAL NETWORK=",mean_shortest_path,"\n",file=paste('FINAL_REPORT_',network_filename,'.txt',sep=""),append=TRUE)
cat(" Average of mean shortest paths of the",number_of_random_networks,"random networks=",mean(mean_shortest_path_random_network),"\n",file=paste('FINAL_REPORT_',network_filename,'.txt',sep=""),append=TRUE)
cat(" Absolute difference between the two values=",abs(mean_shortest_path-mean(mean_shortest_path_random_network)),"\n","---------","\n",file=paste('FINAL_REPORT_',network_filename,'.txt',sep=""),append=TRUE)
cat("\n","---------","\n","Assortativity (r coefficient) of the ORIGINAL NETWORK=",r,"\n","---------","\n",file=paste('FINAL_REPORT_',network_filename,'.txt',sep=""),append=TRUE)
cat("\n","---------","\n","Modularity (Q coefficient) of the ORIGINAL NETWORK=",modcom,"\n","---------","\n",file=paste('FINAL_REPORT_',network_filename,'.txt',sep=""),append=TRUE)
cat("\n","---------","\n","Number of communities found in the ORIGINAL NETWORK=",length(wtc),"\n","---------","\n",file=paste('FINAL_REPORT_',network_filename,'.txt',sep=""),append=TRUE)
cat("\n","---------","\n","γ=",gamma,file=paste('FINAL_REPORT_',network_filename,'.txt',sep=""),append=TRUE)
cat("\n","---------","\n","λ=",lambda,file=paste('FINAL_REPORT_',network_filename,'.txt',sep=""),append=TRUE)
cat("\n","---------","\n","σ(=γ/λ)=",sigma,"\n","---------",file=paste('FINAL_REPORT_',network_filename,'.txt',sep=""),append=TRUE)
if (sigma>1)
{
	cat("\n\n\n","---------","\n","FINAL INFERENCE: The ORIGINAL NETWORK is SMALL WORLD!","\n",file=paste('FINAL_REPORT_',network_filename,'.txt',sep=""),append=TRUE)
}
if (sigma<=1)
{
	cat("\n\n\n","---------","\n","FINAL INFERENCE: The ORIGINAL NETWORK is NOT SMALL WORLD!","\n",file=paste('FINAL_REPORT_',network_filename,'.txt',sep=""),append=TRUE)
}
if (r>0)
{
	cat("                  The ORIGINAL NETWORK is ASSORTATIVE!","\n ---------",file=paste('FINAL_REPORT_',network_filename,'.txt',sep=""),append=TRUE)
}
if (r==0)
{
	cat("                  The ORIGINAL NETWORK is NON-ASSORTATIVE!","\n ---------",file=paste('FINAL_REPORT_',network_filename,'.txt',sep=""),append=TRUE)
}
if (r<0)
{
	cat("                  The ORIGINAL NETWORK is DISASSORTATIVE!","\n ---------",file=paste('FINAL_REPORT_',network_filename,'.txt',sep=""),append=TRUE)
}
system('gnuplot < plot_adjacency_matrix.gnu')
system('gnuplot < plot_knndegree_vs_degree.gnu')
system('gnuplot < plot_lcc.gnu')
system(paste('mv fit.log local_clustering_coefficients_',paste(network_filename,sep=""),'_fitting_details.log',sep=""))
system('gnuplot < plot_mean_clustering_coefficient_of_nodes_with_degree.gnu')
system(paste('mv fit.log mean_clustering_coefficient_of_nodes_with_degree_',paste(network_filename,sep=""),'_fitting_details.log',sep=""))
system('gnuplot < plot_pdf_degrees.gnu')
system(paste('rm -f ',paste('network_average_clustering_coefficients_gnuplot_',network_filename,'.txt',sep=""),sep=""))
#----------------------------------------------------------------------------------------------------------------
# stop the clock
cputime=proc.time()-ptm
#----------------------------------------------------------------------------------------------------------------
cat("\n\n\n","Total user time:",cputime[1],"seconds",file=paste('FINAL_REPORT_',network_filename,'.txt',sep=""),append=TRUE)
cat("\n","System CPU time:",cputime[2],"seconds",file=paste('FINAL_REPORT_',network_filename,'.txt',sep=""),append=TRUE)
cat("\n","Real elapsed time:",cputime[3],"seconds",file=paste('FINAL_REPORT_',network_filename,'.txt',sep=""),append=TRUE)
cat("\n\n\n","Programmed by Dr. Chris G. Antonopoulos",file=paste('FINAL_REPORT_',network_filename,'.txt',sep=""),append=TRUE)
#----------------------------------------------------------------------------------------------------------------
