function spectral_plot(adjacency_matrix_filename)

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
sigma=0.015;
zero_tolerance=1e-10;
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
close all;
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%load the adjacency matrix
cd('..');
A=load(adjacency_matrix_filename);
cd('spectral_plot');
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%degree computations
deg=zeros(length(A),1);
for i=1:length(A)
       for j=1:length(A)
            deg(i)=deg(i)+A(i,j);
       end
end
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%normalized Laplacian computation
L=zeros(length(A),length(A));
for i=1:length(A)
    for j=1:length(A)
         if A(i,j)==1
             L(i,j)=-1/deg(i);
         end
         if i==j
             L(i,j)=1;
         end
    end
end
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% eigenvalues of the normalized Laplacian computation
e=eig(L);
figure(1)
vertical=(1:length(A));
plot(e,vertical,'.');
xlim([0 2]);
ylim([1 length(A)]);
str={adjacency_matrix_filename,'Eigenvalue Spectrum'};
title(str,'Interpreter','none');
xlabel('Eigenvalue');
ylabel('Neuron');
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% smoothing out the eigenvalue spectrum with the Gaussian kernel
xmin=0;
xmax=2;
xnum=2000;
xstep=(xmax-xmin)/xnum;
suma=zeros(xnum+1,1);

for j=1:xnum+1
    x=xmin+(j-1)*xstep;
    for i=1:length(e)
        suma(j)=suma(j)+exp(-((x-e(i))^2)/(2*sigma^2));
    end
    suma(j)=(1/sqrt(2*pi*sigma^2))*suma(j);
end
xcoord=linspace(xmin,xmax,xnum+1);
suma=suma/trapz(xcoord,suma);

figure(2)
plot(xcoord,suma,'r-');
title(str,'Interpreter','none');
str={adjacency_matrix_filename,'Spectral Plot'};
xlabel('Eigenvalue');
ylabel('Relative Frequency');
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% ordered eigenvalues computation
e_ordered=sort(e,'ascend');

% multiplicity of zero eigenvalue computation
mult_zero_eig=0;
for i=1:length(A)
    if e_ordered(i)<=zero_tolerance
        mult_zero_eig=mult_zero_eig+1;
    end
end
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% smallest non zero eigenvalue computation
smallest_non_zero_eigenvalue=e_ordered(mult_zero_eig+1);
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% maximum eigenvalue computation
maximum_eigenvalue=max(e_ordered);
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% eigangaps computation
eigengap=zeros(length(A)-1,1);
for i=1:length(A)-1
    eigengap(i)=e(i+1)-e(i);
end
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% maximum eigengap
[max_eigengap,I]=max(eigengap);
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
dt=datestr(now,'mmmm dd,yyyy HH:MM:SS.FFF AM');
% write in file all relevant variables
fid=fopen('final_results.out','w');
fprintf(fid,dt,'%s');
fprintf(fid,'\nFile processed: %s\n',adjacency_matrix_filename);
fprintf(fid,'\nMultiplicity of zero eigenvalue: %10d',mult_zero_eig);
fprintf(fid,'\nSmallest non-zero eigenvalue: %26.16f',smallest_non_zero_eigenvalue);
fprintf(fid,'\nMaximum eigenvalue: %26.16f',maximum_eigenvalue);
fprintf(fid,'\nMaximum eigengap: %26.16f',max_eigengap);
fprintf(fid,' found in position: %5d',I);
fprintf(fid,'\nSmoothing parameter of the Gaussian Kernel: %26.16f',sigma);
fprintf(fid,'\nTolerance for detecting zero: %26.16f',zero_tolerance);
fprintf(fid,'\nTotal eigenvalue frequency (area of the distribution): %26.16f',trapz(xcoord,suma));
fclose(fid);

fid=fopen('eigenvalues.out','w');
for i=1:length(A)
    fprintf(fid,'%10d %26.16f %26.16f\n',i,e(i),e_ordered(i));
end
fclose(fid);

fid=fopen('degrees.out','w');
for i=1:length(A)
    fprintf(fid,'%10d %10d\n',i,deg(i));
end
fclose(fid);

fid=fopen('normalized_Laplacian.out','w');
for i=1:length(A)
    for j=1:length(A)
        fprintf(fid,'%26.16f',L(i,j));
    end
    fprintf(fid,'\n');
end
fclose(fid);

fid=fopen('eigengaps.out','w');
for i=1:length(eigengap)
    fprintf(fid,'%10d %26.16f\n',i,eigengap(i));
end
fclose(fid);

fid=fopen('spectral_data.out','w');
for i=1:length(suma)
    fprintf(fid,'%26.16f %26.16f\n',xcoord(i),suma(i));
end
fclose(fid);
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
system('gnuplot < plot_spectral_data.gnu');
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

end